/* 
 * Copyright (C) 1998 Peter Zelezny.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <sys/stat.h>
#include <fcntl.h>
#include "gtkutil.h"

/* gtkutil.c, just some gtk wrappers */

typedef void (*filereqcallback) (void *, void *, char *file);
extern void path_part(char *file, char *path);

struct file_req
{
   GtkWidget *dialog;
   void *userdata;
   void *userdata2;
   filereqcallback callback;
   int write;
};

char last_dir[256] = "";

GtkWidget *gtkutil_window_new(char *title, int x, int y, void *callback, gpointer userdata)
{
   GtkWidget *win;

   win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_usize(win, x, y);
   gtk_window_set_title(GTK_WINDOW(win), title);
   gtk_signal_connect(GTK_OBJECT(win), "destroy",
		      GTK_SIGNAL_FUNC(callback), userdata);
   return win;
}

void gtkutil_destroy(GtkWidget *igad, GtkWidget *dgad)
{
   gtk_widget_destroy(dgad);
}

GtkWidget *gtkutil_button_new(char *label, int width, int height, void *callback, gpointer userdata, GtkWidget *box)
{
   GtkWidget *button;

   button = gtk_button_new_with_label(label);
   gtk_widget_set_usize(button, width, height);
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
		      GTK_SIGNAL_FUNC(callback), userdata);
   gtk_container_add(GTK_CONTAINER(box), button);
   gtk_widget_show(button);

   return button;
}

void gtkutil_label_new(char *text, GtkWidget *box)
{
   GtkWidget *label = gtk_label_new(text);
   gtk_container_add(GTK_CONTAINER(box), label);
   gtk_widget_show(label);
}

GtkWidget *gtkutil_entry_new(int max, GtkWidget *box, void *callback, gpointer userdata)
{
   GtkWidget *entry = gtk_entry_new_with_max_length(max);
   gtk_container_add(GTK_CONTAINER(box), entry);
   if(callback)
     gtk_signal_connect(GTK_OBJECT(entry), "changed",
			GTK_SIGNAL_FUNC(callback), userdata);
   gtk_widget_show(entry);
   return entry;
}

GtkWidget *gtkutil_clist_new(int columns, char *titles[],
			     GtkWidget *box, int policy,
			     void *select_callback, gpointer select_userdata,
			     void *unselect_callback, gpointer unselect_userdata)
{
   GtkWidget *clist, *win;

   win = gtk_scrolled_window_new(0, 0);
   gtk_container_add(GTK_CONTAINER(box), win);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(win), 
				  GTK_POLICY_AUTOMATIC,
				  policy);
   gtk_widget_show(win);

   if(titles)
     clist = gtk_clist_new_with_titles(columns, titles);
   else
     clist = gtk_clist_new(columns);
   gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
   gtk_clist_column_titles_passive(GTK_CLIST(clist));
   gtk_container_add(GTK_CONTAINER(win), clist);
   if(select_callback)
   {
      gtk_signal_connect(GTK_OBJECT(clist), "select_row",
			 GTK_SIGNAL_FUNC(select_callback), select_userdata);
   }
   if(unselect_callback)
   {
      gtk_signal_connect(GTK_OBJECT(clist), "unselect_row",
			 GTK_SIGNAL_FUNC(unselect_callback), unselect_userdata);
   }
   gtk_widget_show(clist);

   return clist;
}

GtkWidget *gtkutil_stock_button(GtkWidget *win, char *stock, char *labeltext,
				void *callback, gpointer userdata,
				GtkWidget *box)
{
   GtkWidget *button, *label, *pixmap, *hbox;

   hbox = gtk_hbox_new(0, 0);
   gtk_widget_show(hbox);

   button = gtk_button_new();
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
		      GTK_SIGNAL_FUNC(callback), userdata);
   gtk_widget_show(button);

   pixmap = gnome_stock_pixmap_widget_at_size(win, stock, 12, 14);
   gtk_container_add(GTK_CONTAINER(hbox), pixmap);
   gtk_widget_show(pixmap);

   if(labeltext)
   {
      label = gtk_label_new(labeltext);
      gtk_container_add(GTK_CONTAINER(hbox), label);
      gtk_widget_show(label);
   }

   gtk_container_add(GTK_CONTAINER(button), hbox);
   if(box) gtk_container_add(GTK_CONTAINER(box), button);

   return button;
}

int gtkutil_clist_selection(GtkWidget *clist)
{
   if(GTK_CLIST(clist)->selection)
     return (int)GTK_CLIST(clist)->selection->data;
   else
     return -1;
}
